/****************************************************************************
*                             DMASAMP.C
*     demonstrates continuous measurement using MDD 
*	   and transfer to the PC via DMA.
*	   Part of SORCUS Application Note 067
*
*              version                       : 1.A.
*              creation                      : 16.08.1999
*              last change                   : 16.08.1999
*              author                        : M. Sailer
*
*    (c) by SORCUS Computer GmbH Heidelberg 1999
****************************************************************************/

/****************************************************************************
*  Description: This program measures continuously analog input from 
*        16 channels of the module M-AD12-16 and puts the data into buffers.
*        If the buffer has reached a limit, the task initializes DMA transfer
*		 to the PC.
*
*  Compiler: Microsoft Visual C++ 6.0
*
*  hardware requirements: MODULAR-4/486 board, SPB-module M-AD12-16
*
*  software requirements: This program requires the library MLXW32.LIB
*        and the header MLXW32.H. The operating system OSX on the
*        MODULAR-4/486 board must be ML8-3B.00x or greater
****************************************************************************/

/****************************************************************************
*  history:
*        1.A. 16.08.1999 13:30; MS
*        - creation of the file
****************************************************************************/

/****************************************************************************
*  Includes
****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <conio.h>

// include SORCUS PC library:	Version 7B.004 or greater required!!
#include "mlxw32.h"
// definitions for DMATASK.LIB:
#include "dmatask.h"
// user program running on SORCUS board:
#include "messtask.h"

// data input buffer
USHORT	buffer [512][16];

void	error_handler	(void)
{
	struct ml8_error_info_type  err;
	
	ml8_get_error_info( &err);

   printf("\n\n\n"
			 "************************************************\n");
   printf("************** ERROR OCCURED!!! ****************\n");
   printf("************************************************\n\n");

	switch (err.errortype) 
	{
      case ERROR_DLL:
	         printf("DLL-Error    : \n");
	         printf("errorclass   : %02Xh\n", err.dll.errorclass);
	         printf("errorcode    : %02Xh\n", err.dll.errorcode);
            break;
      case ERROR_COMMUNICATION:
	         printf("Communication Error: \n");
	         printf("errorclass   : %02Xh\n", err.com.errorclass);
	         printf("errorcode    : %02Xh\n", err.com.errorcode);
            break;
      case ERROR_SYSTEM:
	         printf("System Error: %d(d)\n", err.system.error);
            break;
      case ERROR_DRIVER:
	         printf("Driver Error: \n");
	         printf("expected data length: %d(d)\n", 
	                                       err.drv.dataoutsize);
	         printf("real data length: %d(d)\n", 
	                                       err.drv.retlength);
            break;
      default: printf("unknown error\n");
   }
	ml8_clear_error();
}

void	service_proc	(void)
{
	USHORT message;

	message = ml8_get_message();

	printf ("Service request no. %04X\n", message);
}

void	show_error (const CHAR *msg, USHORT errcode)
{
	CHAR errstr [256];

	printf ("Error %s:\n\tCode %04X\n", msg, errcode);

	ml8_get_error_msg (errstr, (UCHAR)(errcode >> 8), (UCHAR)(errcode & 0xff));

	printf ("\t%s\n", errstr);
}

void	main	(void)
{
	USHORT	result, sample_running;
	ULONG		ulNumDMABytes, ulBytesRead, ulNumSamples;

	ml8_bib_startup (LANGUAGE_C);

	// reset SORCUS board and load ROM version of OsX
	result = ml8_reset (0, 0, 1, 0, 0, error_handler, "ROM", 30);

	if (result != 0)
	{
		show_error ("during RESET", result);

		return;
	}

	ml8_set_service (service_proc);

	// install MODULE DEVICE DRIVER for M-AD12-16
	result = ml8_transfer_and_install ("ML8D2700.LIB", 
                                       0, M_AD12_16_SLOT, 0x42C, 0x00, 0x800, 0);
    
   if (result)
	{
		show_error ("loading MDD", result);

		return;
	}

	// check initialization state of MDD
	if (ml8_read_par_byte (M_AD12_16_SLOT, 0) != 0x10)
	{
		printf ("Could not initialize MDD\n");
		return;
	}

	// install DMA driver on SORCUS board
	result = ml8_transfer_and_install ("DMATASK.LIB",
													0, DEFAULT_DMA_TASKNUM, DMATASK_PROGNO,
													0, 0x800, 0);

   if (result)
	{
		show_error ("loading DMATASK.LIB", result);

		return;
	}
	
	// install user program
	result = ml8_transfer_and_install ("MESSTASK.EXE",
													0, MESSTASK_TASKNO, MESSTASK_PROGNO, 
													0, 0x980, 0);

   if (result)
	{
		show_error ("loading MESSTASK.EXE", result);

		return;
	}

	// start MESSTASK.EXE
	ml8_call_proc (MESSTASK_TASKNO, 2);

	// read parameter.usStatus
	result = ml8_read_par_word (MESSTASK_TASKNO, 0);

	if (result != RUNNING)
	{
		printf ("Could not start MESSTASK.EXE\n");
		return;
	}

	sample_running = 1;

	while (sample_running)
	{
		if (kbhit ())
		{
			switch (getch())
			{
			case 0x1B: // ESC pressed
				sample_running = 0;
				break;
			case 'b': // read number of bytes transmitted so far
			case 'B':
				ml8_read_par_dword (MESSTASK_TASKNO, 2, &ulNumDMABytes);
				printf ("Bytes transmitted: %d\n", ulNumDMABytes);
				break;
			case 's':
			case 'S':
				ml8_read_par_dword (MESSTASK_TASKNO, 6, &ulNumSamples);
				printf ("Number of samples: %d\n", ulNumSamples);
				break;
			}
		}
		
		result = ml8_read_dma (buffer, sizeof(buffer), &ulBytesRead);

		if (result)
		{
			show_error ("reading data", result);
		}
		else
		{
			printf ("received %d bytes\n", ulBytesRead);
		}
	}

	// stop MESSTASK.EXE
	ml8_call_proc (MESSTASK_TASKNO, 3);
}





				
