/****************************************************************************
*                             MESSTASK.C
*      demonstrates continuous measurement using MDD 
*	   and transfer to the PC via DMA.
*	   Part of SORCUS Application Note 067
*
*              version                       : 1.A.
*              creation                      : 16.08.1999
*              last change                   : 16.08.1999
*              author                        : M. Sailer
*
*    (c) by SORCUS Computer GmbH Heidelberg 1999
****************************************************************************/
 
/****************************************************************************
*  Description: This program measures continuously analog input from 
*        16 channels of the module M-AD12-16 and puts the data into buffers.
*        If the buffer has reached a limit, the task initializes DMA transfer
*		 to the PC.
*
*  Compiler: Borland C 3.1
*
*  hardware requirements: MODULAR-4/486 board, SPB-module M-AD12-16
*
*  software requirements: This program requires the library ML8RTBIB.LIB
*        and the header ML8RTBIB.H. The operating system OSX on the
*        MODULAR-4/486 board must be ML8-3B.00x or greater
****************************************************************************/

/****************************************************************************
*  history:
*        1.A. 16.08.1999 13:30; MS
*        - creation of the file
****************************************************************************/

/****************************************************************************
*  Includes
****************************************************************************/

#include <dos.h>		       /* Include the default C libraries  */

#include "ml8rtbib.h"                  /* header of real-time-library      */
				       /* requires Turbo-Assembler         */

#include "DMATASK.H"
#include "messtask.h"                   /* Include the header file          */

#define DMA_BUFFER_SIZE	16384

/****************************************************************************
*  Defines
****************************************************************************/

#define PROGRAM_NUMBER  0x402          /* program-number = 402h            */
#define VERSION         '1'            /* program-version                  */
#define REVISION        'A'            /* program-revision                 */

				       /* global values                    */
par_type_MESSTASK parameter;            /* Parameters of program MESSTASK    */

void PFAR main_task(void);                  /* prototypes of the procedures     */
void PFAR auto_init(void);
void PFAR start(void);
void PFAR stop(void);

/****************************************************************************
*  auto_init
*        initializes the parameters of the program
*  parameter:     none
****************************************************************************/

void PFAR auto_init(void)                   /* AUTO-INIT-PROCEDURE              */
{
	ml8rt_entry();                        /* save & prepare registers         */

 	parameter.usStatus = INSTALLED;         /* set status = 0                   */

 	ml8rt_exit();                         /* rebuild registers                */
}

/****************************************************************************
*  start
*        creates the system buffer used for storing the analog input values
*        and activates the TI- and NI-Task
*  parameter:     none
****************************************************************************/

void PFAR start(void)                       /* START-PROCEDURE                  */
{
	USHORT i;
	ULONG size;
	CPS_MAD1216 cps;

	ml8rt_entry();                        /* save & prepare registers         */

	cps.usDevice = DEVICE_AIN_SE;
	cps.usIndexFirst = 0;
	cps.usIndexLast  = 15;
	cps.usFlags = _CP_EXCLUSIVE;
	cps.usReadMode = IO_MODE_DIRECT;
	cps.usRange = RANGE_UPP_5V;

	parameter.hMDD =
			mdd8_open_channel (M_AD12_16_SLOT, sizeof(CPS_MAD1216), &cps);

	if (!parameter.hMDD)
	{
		parameter.usStatus = MDD_ERROR;
		return;
	}

	parameter.usBufferNum = 0;

	parameter.usBufferIndex [0] = parameter.usBufferIndex [1] = 0;

	parameter.usBufferState [0] = parameter.usBufferState [1] = 0;

	if (parameter.usStatus == INSTALLED)
	{
		for (i=0;i<2;i++)
		{
			size = DMA_BUFFER_SIZE;

			parameter.rcBufferList [i].pNext = 0;

			parameter.rcBufferList [i].ulSize = DMA_BUFFER_SIZE;

			ml8rt_allocate_ram (parameter.tdt.task, 0, ALLOC_UP_ABS, WORD_ALIGN, &size, &parameter.rcBufferList [i].pData);

			if (size != DMA_BUFFER_SIZE)
			{
				// not enough free memory
				parameter.usStatus = MEMORY_ERROR;
				return;
			}

			if (parameter.rcBufferList [i].pData > 0x100000l - DMA_BUFFER_SIZE)
			{
				// memory buffer is not completely in REAL MODE memory space
				parameter.usStatus = MEM_RANGE_ERROR;
				return;
			}

			// calculate SEG:OFFS address of memory buffer
			ml8rt_real_adr (parameter.rcBufferList [i].pData, &parameter.pucMemoryBuffer [i]);
    	}
	}
	else if (parameter.usStatus != CANCELLED)
	{
		return;
	}

 	ml8rt_wakeup_ti_task(parameter.tdt.task, 0, 0, 1, 0l);

	parameter.usStatus = RUNNING;

    ml8rt_exit();                         /* rebuild registers                */
}

/****************************************************************************
*  stop
*        deactivates the TI- and the NI-Task and sets the status = 4
*  parameter:     none
****************************************************************************/

void PFAR stop(void)                        /* STOP-PROCEDURE                   */
{
 	ml8rt_entry();                        /* save & prepare registers         */
									       /* deactivates the TI-Task          */
 	ml8rt_sleep_task(parameter.tdt.task);
 	
	parameter.usStatus = CANCELLED;         /* status = 4 (cancelled)           */

	if (parameter.hMDD)
	{
		mdd8_close_channel (&parameter.hMDD);
	}

 	ml8rt_exit();                         /* rebuild registers                */
}


void  PutDMARequestToPC (void)
{
	USHORT index, outsize, error, insize;
	DMAREQUEST dmarq;

	index = parameter.usBufferNum;

	if (index == 0xFFFF)
		return;

	ml8rt_phys_adr (&parameter.rcBufferList [index], &dmarq.pBufList);

	dmarq.usCallbTask = parameter.tdt.task;
	dmarq.usCallbFunc = 4; // Task number of function 'DMACallback'
	dmarq.ulCallbParam = index;

	insize = sizeof (DMAREQUEST);

	parameter.usBufferState [index] = BUFFERSTATE_TRANSFER;

	ml8rt_call_func (DEFAULT_DMA_TASKNUM, REQUEST_TOPC, &insize, &dmarq, 0, &outsize,
		&outsize, &error);

	if (error == 0)
	{
		index = (index == 0) ? 1 : 0;

		if (parameter.usBufferState [index] == BUFFERSTATE_FREE)
 	    {
			parameter.usBufferState [index] = BUFFERSTATE_IN_USE;
			parameter.usBufferNum = index;
			parameter.usBufferIndex [index] = 0;
			return;
		}
	}

	// transfer of previous buffer has obviously not been finished yet
	ml8rt_send_buffer_srq (SRQWORD_TRANSFER_ERROR, 1);
	parameter.usBufferNum = 0xFFFF;
}

/****************************************************************************
*  DMACallback
****************************************************************************/

void far pascal DMACallback (USHORT task, USHORT insize,
			  void far *indata, USHORT outsize, void far *outdata)
{
	DMARESULT *pCallback;
	USHORT error, index;

	ml8rt_entry_function();       /* save and prepare registers       */

	error = 0;

	pCallback = (DMARESULT *)indata;

	index = pCallback->ulCallbParam;

	parameter.usBufferState [index] = BUFFERSTATE_FREE;

	parameter.ulNumDMABytes += pCallback->ulNumBytes;

	if (pCallback->usReason != DMASRQ_COMPLETE &&
	    pCallback->usReason != DMASRQ_NOMOREBUFFERS)
	{
		ml8rt_send_buffer_srq (SRQWORD_TRANSFER_ERROR, 1);
	}
	
	ml8rt_exit_function(0, outsize, error);
}

/****************************************************************************
*  main_task
*        This main procedure is called by the TI-task scheduler of the
*        operating system OsX. It converts 16 A/D-channels and stores the
*        data into a ring buffer of the OsX.
*  parameter:     none
****************************************************************************/

void PFAR main_task(void)                   /* MAIN-PROCEDURE of TI-Task        */
{
	USHORT index, numbytes;

 	ml8rt_entry();                        /* save & prepare registers         */

	index = parameter.usBufferNum;

	if (index != 0xFFFF)
	{
		numbytes = mdd8_read_channel_block (parameter.hMDD, 32, 
					 parameter.pucMemoryBuffer [index] +
					 parameter.usBufferIndex [index]);

		if (numbytes == 32)
		{
			parameter.ulNumSamples += 16;
	
			parameter.usBufferIndex [index] += 32;

			if (parameter.usBufferIndex [index] >= DMA_BUFFER_SIZE)
			{
				PutDMARequestToPC ();
			}
		}
		else
		{
			ml8rt_send_buffer_srq (SRQWORD_MDD_ERROR, 1);
		} 
	}
 	ml8rt_exit();                         /* rebuild registers                */
}

void main ()                           /* PREPARE                          */
{
 	pdt.typ = 1;                          /* set PDT-type (only type 1 is     */
				       					  /* supported at the moment)         */
	pdt.lengt      = _PDT_LENGTH;         /* length of PDT-header             */
				       					  /* (at the moment always = 48)      */
	pdt.proc_num   = 5;                   /* number of procedures             */
	pdt.prg_number = PROGRAM_NUMBER;      /* program-number                   */
 	pdt.version    = VERSION;             /* program-version                  */
 	pdt.revision   = REVISION;            /* program-revision                 */
 	pdt.cpu_type   = _486DX;              /* CPU-type                         */
 	pdt.coproz_type = _487DX;             /* Co-processor-type                */
 	pdt.language   = _CPP31;              /* programming language = C++       */
 	pdt.prg_type   = _USER_PRG;           /* program-type: user-program       */
 	pdt.flags      = 0;                   /* flags                            */
 	pdt.flags      = _TI_TASK +
		  			 _PDT_INFO_ENABLE +
		  			 _LOCAL_DATA +
		  			 _LOCAL_PARAMETER +
		  			 _FIXED_DATASIZE;
    pdt.interrupt_num = IRQ_TIMER_C;      /* interrupt-number for this prg.   */
				       					  /* (Timer-C = 93h, TI-TASK)         */

 	pdt.data_adr   = 0;                   /* address of data-area             */
	 			       					  /* (not used here)                  */
 	pdt.data_size  = 0;                   /* size of data-area                */
 	pdt.data_min   = 0;                   /* minimum size of data-area        */
 	pdt.data_max   = 0;                   /* maximum size of data-area        */
				       					  /* calculate                        */
				       					  /* address of parameter-area        */
 	ml8rt_phys_adr (&parameter, &pdt.parameter_adr);
 	pdt.parameter_adr  = pdt.parameter_adr + sizeof(tdt_type);
				       					  /* set number of parameters         */
 	pdt.parameter_size = sizeof(parameter) - sizeof(tdt_type);
 	pdt.hyper_adr = 0;                    /* addr. of hypertext (not used)    */

 	pdt.main_proc = main_task;            /* addr. of main-procedure          */
 	pdt.auto_init = auto_init;            /* addr. of Auto-Init-proc.         */
 	pdt.start     = start;                /* addr. of Start-procedure         */
 	pdt.stop      = stop;                 /* addr. of Stop-procedure          */
 	pdt.callback  = DMACallback;          /* addr. of special procedure       */
 	ml8rt_set_pdt_adr(&pdt);              /* operating system needs the       */
				  					      /* address of the PDT               */
}




